<?php
#######################################################################################
#
# container-klasse fr sessions
#
#######################################################################################
#
# autor    Michael Brauchl
# datum    18.04.2001
# version  0.0.1
# lizenz   GPL
#
#######################################################################################
#
# void container
# ==============
# intitiert die session, checkt, ob das session-directory
# vorhanden ist und ob es sich um ein directory handelt.
#
# bool check_session(ID START_TIME, LIFE_TIME)
# ============================================
# berprft ob es einen container mit dieser ID gibt
# und ob die session noch nicht abgelaufen ist.
#
# STRING         ID         = session-id
# UNIX TIMESTAMP START_TIME = zeit an der die momentane session gestartet wurde.
# UNIX TIMESTAMP LIFE_TIME  = maximale lebenszeit einer session in sekunden.
#
# void import_vars(ID, VARS)
# ==========================
# importiert die werte aus dem container in die bergebene variable VARS. VARS wird 
# als zeiger bernommen.
#
# STRING ID   = session-id
# MIXED  VARS = variable der session-klasse 
#
# void update_time(ID)
# ====================
# ndert den zeitstempel des containers auf jetzt.
#
# STRING ID   = session-id
#
# void delete(ID)
# ===============
# zerstrt die daten im container
#
# STRING ID   = session-id
#
# void collect_garbage(START_TIME, GLOBAL_EXPIRES)
# ================================================
# testet ob es container gibt, die die maximale lebensdauer von containern 
# berschritten haben. gibt es solche container werden sie zerstrt.
# 
# UNIX TIMESTAMP START_TIME     = zeit an der die momentane session gestartet wurde.
# UNIX TIMESTAMP GLOBAL_EXPIRES = maximale lebenszeit einer session in sekunden.
#
# void write_vars(ID, VARS)
# =========================
# schreibt die variablen in VARS in den container.
#
# STRING ID   = session-id
# MIXED  VARS = variable der session-klasse
#
#######################################################################################
#
# neu: erwartet $PHPCMS_INCLUDEPATH auf den include-path von phpCMS gesetzt!
#
#######################################################################################

if(!defined("_SESSION_FILE_"))  
	define("_SESSION_FILE_",TRUE);

if (!defined("session_dir")) 
	define ('session_dir', $PHPCMS_INCLUDEPATH.'/../session');

class container
	{
	var $file_pointer;
	
	function check_dir()
		{
		debug_lines('check_dir');
		if ( !file_exists(session_dir))
			print_error ('directory for storing sessions: "'.session_dir.'" does not exists.', __FILE__, __LINE__);
		if ( !is_dir(session_dir))
			print_error ('directory for storing sessions: "'.session_dir.'" is not a directory.', __FILE__, __LINE__);
		return;
		}
		
	function check_session($ID, $start_time, $life_time)
		{
		# Testet ob die Session mit der ID
		# eine aktive Session ist oder nicht
		
		debug_lines('make_container');
		
		# Wenn eine Datei mit dieser Session-ID existiert
		if ( file_exists(session_dir.'/'.$ID))
			{
			# Cache lschen
			clearstatcache();
			# Wenn Session vorhanden aber abgelaufen
			if ( $start_time > (filemtime(session_dir.'/'.$ID) + $life_time) )
				{
				unlink (session_dir.'/'.$ID);
				return false;
				}
			return true;
			}
		return false;
		}
		
	function import_vars($ID, &$vars)
		{
		# Diese Funktion importiert die Werte
		# aus dem Inhalt einer session in das
		# array session->vars.
		debug_lines('import_vars');
		
		$var_array = file(session_dir.'/'.$ID);
		$count_var_array = count($var_array);
		for ($i=0; $i< $count_var_array; $i++)
			{
			list ($key, $value) = explode('||',$var_array[$i]);
			$vars["$key"] = unserialize ($value);
			}
						
		return;
		}
		
	function write_vars($ID, $vars)
		{
		# diese funktion schreibt die werte
		# einer session in das container-objekt
		debug_lines('write_vars');
		
		if ( isset ( $vars ) )
			{
			$this->file_pointer=fopen(session_dir.'/'.$ID, 'w+');
			while (list ($key, $value) = each($vars)) 
				{
				$writer = serialize ($value);
				fwrite ($this->file_pointer, $key.'||', strlen($key)+2);
				fwrite ($this->file_pointer, $writer."\n", strlen($writer)+1);
				}
			fclose ($this->file_pointer);
			}
		}
		
	function collect_garbage($start_time, $global_expires)
		{
		# diese funktion wird am ende jeder session
		# aufgerufen und sieht nach, ob abgelaufene
		# container-objekte im session-dir
		# sind und lscht diese bei bedarf.
		debug_lines('collect_garbage');
		
		$d = dir (session_dir);
		while($entry=$d->read()) 
			{
			if ($entry == '.') continue;
			if ($entry == '..') continue;
			if($start_time > (filemtime(session_dir.'/'.$entry) + $global_expires) )
				unlink (session_dir.'/'.$entry);
			}
		$d->close();      
		}
	
	function update_time($ID)
		{
		@touch(session_dir.'/'.$ID);
		}
	
	function delete($ID)
		{
		# lscht container-objekt
		debug_lines('delete');
		unlink (session_dir.'/'.$ID);
		}
		
	function container()
		{
		# es wird getestet, ob das session-Verzeichnis
		# vorhanden ist.
		debug_lines('container');
		$this->check_dir();		
		}		
	}
?>