<?php 

/**
* Basisklasse fr die Datenhaltung.
* Es stehen entsprechende Grundfunktionen bereit, die
* einfach 'ererbt' werden knnen.
* Im vordefinierten, allerdings leeren Container (array)
* internals werden die zu speichernden Daten definiert und abgelegt.
*
* Eine neue ableitung kann/sollte folgendes Format haben :
*
* class ooo extends DataStoreBasic {
*   var $internals = array (
*         'DATENSATZ_1' => '',
*         'DATENSATZ_2' => '',
*         'DATENSATZ_3' => '' )
*   }
*
* Da bereits entsprechende Funktionen definiert sind, wrde dies zur
* Datenhaltung ausreichen.
*
* @author    Michael Taupitz <mitaco@gmx.net>
*
*/

class DataStoreBasic {

	/**
	* Datencontainer (array) Dieser sollte nicht
	* direkt angesprochen werden, sondern nur durch
	* die vorhanden Methoden get_var() und set_var().
	* Es werden 3 standard-parameter gesetzt.
	* ID 			- Eindeutige Kennzeichnung des Datensatzes
	* CREATE_TIME 	- Zeitpunkt der Erzeugung ( create_id() )
	* LAST_MOD  	- Zeitpunkt der nderung/Speicherung ( save_data() )
	*
	* @see get_var(), set var(), get_store(), put_store()
	* @var Array
	* @access private
	*/
  	var $internals = array( 'ID' => '' , 'CREATE_TIME' => '', 'LAST_MOD' => '' );
  	
  	/**
  	* Hier wird die Instanz der Klasse zur Daten-Speicherung
  	* als Class-Member abgelegt.
	*
	* @see data_container::data_container()
	* @var Object
	* @access private
	*/  	
	var $data_container;

  	/**
  	* Bezeichnet ob das interne Array der Daten beliebig durch
  	* 'unbekannte' Bezeichner beim set_var() erweitert werden
  	* drfen.
	*
	* @see set_var()
	* @var Boolean
	* @access private
	*/  	
	var $strict;


	/**
	* Konstruktor
	* Initiert die Instanz nd erstellt auch eine eigene ID
	*
	* @parameter String $datadir Daten-Verzeichnis
	* @parameter Boolean $strict Bezeichnet ob beim setzen von Daten auch neue Container
	*                            automatisch angelegt werden.
	*
	* @access public
	*/
	
	function DataStoreBasic($datadir,$strict = false)
	{		
		$this->data_container = new data_container($datadir);	
		$this->create_id();
		$this->strict = $strict;
	}

	/**
	* Erstellt eine neue ID
	*
	* @return 
	* @access public
	*/

	function create_id()
	{		
		$this->set_var('ID', md5 (uniqid (rand())));			
	}

	/**
	* Dient zum Suchen von Daten und ruft die
	* entsprechende Funktion in der data_container-
	* Klasse auf.
	*
	* @parameter String $KEY Tabelle bzw. Bezeichnung in der gesucht wird
	* @parameter String $VALUE Wert der gesucht wird
	*
	* @return Array Ein Array der zu dem Suchbegriff gefundenen IDs
	*
	* @see data_container::search_data()
	* @access public
	*/
	function search_data( $KEY, $VALUE )
	{			
		return $this->data_container->search_data($KEY, $VALUE);		
	}

	/**
	* Ldt die zu der angegebenen ID gehrenden Daten
	* in den internen Daten-Container.
	*
	* @parameter String $ID ID
	*
	* @see $internals, data_container::read_data()
	* @access public
	*/
	
	function get_data($ID)
	{
		if (!isset($ID)) {
			print_error ('no id in call!', __FILE__, __LINE__);
		} else {
			reset($this->internals);
			while(list($key,$val) = each($this->internals))
				$this->set_var($key,  $this->data_container->read_data($ID,$key) );
		}
	}
	
	/**
	* Speichert die im Datencontainer liegenden Daten
	* unter der angegebenen ID ab.
	*
	* @parameter String $ID ID
	*
	* @see $internals, data_container::write_data()
	* @access public
	*/

	function save_data($ID)
	{
		if (!isset($ID)) {
			print_error ('no id in functioncall!', __FILE__, __LINE__);
		} else {		

			//Store the Last Modification-Time.
			$this->set_var('LAST_MOD',time());				

			reset($this->internals);
			while(list($key,$val) = each($this->internals))
				$this->data_container->write_data($ID,$key,$this->internals[$key]);
		}
	}
		

	/**
	* Lscht die zur ID gehrenden Daten.
	*
	* @parameter String $ID ID
	*
	* @see $internals, data_container::delete_data()
	* @access public
	*/

	function delete_data($ID)
	{		
		if (!isset($ID)) {
			print_error ('no id in functioncall!', __FILE__, __LINE__);
		} else {
			reset($this->internals);
			while(list($key,$val) = each($this->internals))
				$this->data_container->delete_data($ID,$key);
		}

	}

	/**
	* Holt Daten zu der angegebenen Bezeichnung aus
	* dem Datencontainer.
	*
	* @parameter String $key Name des Datencontainers
	*
	* @return MIXED Inhalt des Datencontainers
	*
	* @see $internals
	* @access public
	*/

	function get_var($key)
	{
		if (isset($this->internals[$key])) {
			return $this->internals[$key];
		} else {
			return '';	
		}
	}

	/**
	* Speichert Daten unter der angegebenen Bezeichnung in
	* dem Datencontainer.
	*
	* @parameter String $key Name des Datencontainers
	* @parameter MIXED  $val Zu speichernde Daten
	*
	* @see $internals
	* @access public
	*/

	function set_var($key,$val)
	{
		if (isset($key) && isset($val)) {
			if (isset($this->strict) && ($this->strict == true) && isset($this->internals[$key])) {							
				$this->internals[$key] = $val;
			} else {
				$this->internals[$key] = $val;
			}			
		}
	}
}


/**
* Klasse zu Speicherung aller zu einer Seite
* gehrenden Daten.
*
* @author    Michael Taupitz <mitaco@gmx.net>
*/

class phpCMSSiteInfo extends DataStoreBasic{

	/**
	* Datencontainer (array) Dieser sollte nicht
	* direkt angesprochen werden, sondern nur durch
	* die vorhanden Methoden get_var() und set_var().
	*
	* @see DataStoreBasic::get_var(), DataStoreBasic::set var()
	* @var Array
	* @access private
	*/

	var $internals = array(
		'SITEINFO_FILENAME' => '',
		'SITEINFO_EDITPASSWORD' => '',
		'SITEINFO_MENU' => '',
		'SITEINFO_MENUTEMPLATE' => '',
		'SITEINFO_TEMPLATE' => '',
		'SITEINFO_TAGS' => '',
		'SITEINFO_STORE' => array()
	);
	
	var $strict;
	
	/**
	* Konstruktor
	* Initiert die Instanz und ruft den Konstruktor
	* der Basisklasse auf.
	*
	* @see DataStoreBasic::DataStoreBasic()
	* @parameter String $datadir Daten-Verzeichnis
	* @parameter Object $project Refferenz auf ein Project, dessen Deafult-Einstellungen bernommen werden.
	* @access public
	*/

	function phpCMSSiteInfo($datadir, &$project)
	{
		$this->DataStoreBasic($datadir);
		//Default-Werte setzen
		$this->set_var('SITEINFO_EDITPASSWORD',	$project->get_var('PROJECT_DEFAULTPASSWORD'));
		$this->set_var('SITEINFO_MENU',			$project->get_var('PROJECT_DEFAULTMENU'));
		$this->set_var('SITEINFO_MENUTEMPLATE',	$project->get_var('PROJECT_DEFAULTMENUTEMP'));
		$this->set_var('SITEINFO_TEMPLATE',		$project->get_var('PROJECT_DEFAULTTEMPLATE'));
		$this->set_var('SITEINFO_TAG',			$project->get_var('PROJECT_DEFAULTTAG'));	
	}

}	

/**
* Klasse zu Speicherung und Verwaltung von
* Projekten. 
*
* @author    Michael Taupitz <mitaco@gmx.net>
*/

class phpCMSProject extends DataStoreBasic {

	/**
	* Datencontainer (array) Dieser sollte nicht
	* direkt angesprochen werden, sondern nur durch
	* die vorhanden Methoden get_var() und set_var().
	*
	* @see DataStoreBasic::get_var(), DataStoreBasic::set var()
	* @var Array
	* @access private
	*/
	
	var $internals = array(
		'PROJECT_NAME' => '',
		'PROJECT_DIR' => '',
		'PROJECT_GFX_DIR' => '',
		'PROJECT_PLUGIN_DIR' => '',
		'PROJECT_USER_DIR' => '',
		'PROJECT_GROUP_DIR' => '',
		'PROJECT_TEMPLATE_DIR' => '',
		'PROJECT_LANGUAGE' => '',

		'PROJECT_DEFAULTPASSWORD' => '', //$DEFAULTS->get_var('EDITPASSWORD'),
		'PROJECT_DEFAULTTEMPLATE' => '', //$DEFAULTS->get_var('TEMPLATE'),
		'PROJECT_DEFAULTMENU' => '', //$DEFAULTS->get_var('MENU'),
		'PROJECT_DEFAULTTAG' => '', //$DEFAULTS->get_var('TAGS'),
		'PROJECT_DEFAULTMENUTEMP' => '', //$DEFAULTS->get_var('MENUTEMP'),

		'PROJECT_SITES' => array(),
		
		'PROJECT_STORE' => array()		
	);

	var $strict;

	/**
	* Konstruktor
	* Initiert die Instanz und ruft den Konstruktor
	* der Basisklasse auf.
	*
	* @see DataStoreBasic::DataStoreBasic()
	* @parameter String $datadir Daten-Verzeichnis
	* @parameter Object $project Refferenz auf ein Project, dessen Deafult-Einstellungen bernommen werden.
	* @access public
	*/

	function phpCMSProject($datadir)
	{	
		$this->DataStoreBasic($datadir);
	}
	
	/**
	* Dient zum Suchen eines Projektes.
	*
	* @parameter String $KEY Tabelle bzw. Bezeichnung in der gesucht wird
	* @parameter String $VALUE Wert der gesucht wird
	*
	* @return Array Ein Array der zu dem Suchbegriff gefundenen IDs
	*
	* @see DataStoreBasic:search_data(),data_container::search_data()
	* @access public
	*/

	function search_project( $KEY, $VALUE )
	{
		if (! isset($VALUE))
			print_error ('no field in functioncall! (VALUE='. $VALUE .')', __FILE__, __LINE__);					      						
		if (! isset($KEY))
			print_error ('no field in functioncall! (KEY='. $KEY .')', __FILE__, __LINE__);					      						
		return $this->data_container->search_data($KEY, $VALUE);		
	}


	/**
	* Ldt die zu der angegebenen ID gehrenden Daten
	* in den internen Daten-Container.
	*
	* @parameter String $ID ID
	*
	* @see $internals, DataStoreBasic:get_data(),data_container::read_data()
	* @access public
	*/

	function get_project_data($ID)
	{
		if (!isset($ID)) {
			print_error ('no project-id in call!', __FILE__, __LINE__);
		} else {
			$this->get_data($ID);
		}
	}
	
	/**
	* Speichert die Daten unter der angegebenen ID.
	*
	* @parameter String $ID ID
	*
	* @see $internals, DataStoreBasic:save_data(),data_container::write_data()
	* @access public
	*/

	function save_project_data($ID)
	{
		if (!isset($ID))
			print_error ('no project-id in functioncall!', __FILE__, __LINE__);				
		$this->save_data($ID);
	}		


	/**
	* Lscht die zur ID gehrenden Daten.
	*
	* @parameter String $ID ID
	*
	* @see $internals, DataStoreBasic:delete_data(),data_container::delete_data()
	* @access public
	*/

	function delete_project($ID)
	{
		if (!isset($ID))
			print_error ('no project-id in functioncall!', __FILE__, __LINE__);

		$this->delete_data($ID);
		}


	/**
	* Fgt die ID einer SiteInfo dem Projekt zu.
	*
	* @parameter String $ID ID
	* @access public
	*/
	
	function add_siteinfo($ID)
	{	
		$sitearray = $this->get_var('PROJECT_SITES');
		$sitearray[] = $ID;
		$this->set_var('PROJECT_SITES',$sitearray);
	}		

	/**
	* Entfernt eine SiteInfo-ID aus dem Projekt.
	* Achtung - keine Physikalische lschung !
	*
	* @parameter String $ID ID
	* @access public
	*/

	function remove_siteinfo($ID)
	{		
		$sitearray = $this->get_var('PROJECT_SITES');
		for ($ii=0;$ii<count($sitearray);$ii++)
		{
			if ($sitearray[$ii] == $ID) {
				unset($sitearray[$ii]);
				break;
			}
		}				
		$this->set_var('PROJECT_SITES',$sitearray);
	}		


	/**
	* Gibt die Liste von IDs der zum Projekt gehrenden 
	* SiteInfos.
	*
	* @return Array Array von SiteInfo-IDs 
	* @access public
	*/

	function get_siteinfo_ids()
	{
		return $this->get_var('PROJECT_SITES');
	}		



	/**
	* bernahme bestehender Projekte.
	* Es wird das angegebene Verzeichnis und alle
	* Unterverzeichnisse durchsucht und die zur
	* extension passenden Dateien als Content-Dateien
	* bernommen.
	* In jeder Content-Datei wird nachgesehen welche Projektdatei
	* zustndig ist, und die Daten hieraus ausgelesen.
	* Die gefundenen SiteInfos werden gespeichert, und in die
	* Listes dieses Projektes bernommen.
	* Achtung : Das Projekt wird nicht automatisch gespeichert.
	*
	* @parameter String $dirname Verzeichnis von dem an gesucht werden soll
	* @parameter String $ext Extension der Content-Dateien
	*
	* @access public
	*/

	
	/*
	* Diese Funktion soll bestehende Projekt in die Projekt-Struktur einlesen.
	*/
		
	function revert_dir($dirname,$ext=".htm",$basedir="")	
	{
		global $PHP,$DEFAULTS;
		
		if ($basedir=="")
			$basedir = $dirname;

		// try to figure out what delimeter is that's already in use... 
		$delim = (strstr($dirname,"/")) ? "/" : "\\"; 

		if($dirname[strlen($dirname)-1]!=$delim) 
			$dirname.=$delim; 

		echo "revert_dir (".$dirname.")<br>";

		$handle = opendir($dirname); 
	
		while ($file = readdir($handle))
		{		

			if($file=='.'||$file=='..') 
				continue; 

			if(is_dir($dirname.$file))
			{
				$this->revert_dir($dirname.$file.$delim,$basedir);
			}
			else
			{
				//nur wenn die extension stimmt.
				if (stristr($dirname.$file,$ext))
				{
					echo "revert: " . $dirname.$file ."<br>";
					$me_site = new phpCMSSiteInfo($PHP->GetDocRoot().$PHP->GetScriptPath().'/' . $DEFAULTS->PROJECT_DIR,$this);
					$me_site->set_var('SITEINFO_FILENAME', $basedir . $file);										
					
					$content = simpleParse($dirname.$file);
					if (isset($content['PROJECT']))
					{
						$info = file($PHP->GetDocRoot().$this->get_var('PROJECT_DIR') . '/' .	trim($content['PROJECT']));
						for ($ii=0;$ii<count($info);$ii++)
						{
							
							if ( $info[$ii][0] != ';' && stristr ( $info[$ii], ':=' ) )
							{
								$temp = split (":=", $info[$ii]);
								if (count($temp) > 1)
								{
								    echo trim($temp[0]) . "=" . trim($temp[1]) . "<br>";
									$me_site->set_var('SITEINFO_' . strtoupper(trim($temp[0])),trim($temp[1]));
								}
							}
						}
					}
					$this->add_siteinfo($me_site->get_var('ID'));
					
					$me_site->save_data($me_site->get_var('ID'));

					echo $me_site->get_var('ID') . '(' . $me_site->get_var('SITEINFO_FILENAME') .')<br>';
				}						
			}//else
		} //while																					
	}
		


	/**
	* Speichert Daten in dem STORE-Bereich des Projektes
	*
	* @parameter String $key Name des Datencontainers
	* @parameter String $key Daten die gespeichert werden sollen
	*
	* @see $internals
	* @access public
	*/

	function put_store($key,$val)
	{
		$storearray = $this->get_var('PROJECT_STORE');
		$storearray[$key] = $val;
		$this->set_var('PROJECT_STORE',$storearray);
	}

	/**
	* Holt Daten zu dem STORE-Bereich des Projektes
	*
	* @parameter String $key Name des Datencontainers
	*
	* @return MIXED Inhalt des Datencontainers
	*
	* @see $internals
	* @access public
	*/

	function get_store($key)
	{
		$storearray = $this->get_var('PROJECT_STORE');
		return $storearray[$key];
	}
		
}
?>